
/* **************************************************************************
  $Archive: /njcl_v2/src/com/novell/service/nds/net/NetBoolean.java $
  $Revision: 6 $
  $Modtime: 1/28/00 12:46p $
 
  Copyright (c) 1998 Novell, Inc.  All Rights Reserved.

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.

****************************************************************************/

package com.novell.service.nds.net;


import java.io.*;

import com.novell.java.io.EndianInputStream;
import com.novell.java.io.EndianOutputStream;

import com.novell.service.jncp.NSIException;

import com.novell.service.nds.NdsBoolean;


/** @internal
 * NetBoolean is an Adapter for NdsBoolean.
 *
 * This class converts to/from the NWNet Boolean_T.
 *
 * API Data Structure:
 *    typedef nuint8 Boolean_T;
 */
public final class NetBoolean
   extends NdsBoolean implements NetAttributeValue, Serializable
{
   /**
    * Constructs a Boolean from the "API Data Structure".
    *
    * @param   value   byte[] containing a Boolean_T
    */
   protected NetBoolean (
         byte[] value)
      throws NSIException
   {
      try
      {
         EndianInputStream edis = new EndianInputStream (value);

         this.value = (edis.readLoHiInt () == 0) ? false : true;
      }
      catch (IOException e)
      {
         NSIException ne = new NSIException ();

         ne.setRootCause (e);
         throw (ne);
      }
   }

   /**
    * Constructs a NetBoolean adapter for an NdsBoolean.
    *
    * @param   value   an NdsBoolean object.
    */
   protected NetBoolean (
         NdsBoolean value)
   {
      super (value);
   }

   /**
    * Returns a Boolean "API Data Structure".
    *
    * @return   byte[] containing Boolean_T.
    */
   public byte[] toByte ()
      throws NSIException
   {
      try
      {
         ByteArrayOutputStream baos = new ByteArrayOutputStream ();
         EndianOutputStream eos = new EndianOutputStream (baos);

         eos.writeLoHiInt ((this.value == false) ? 0 : 1);
         return (baos.toByteArray ());
      }
      catch (IOException e)
      {
         NSIException ne = new NSIException ();

         ne.setRootCause (e);
         throw (ne);
      }
   }

} /* NetBoolean */


